@@ -1,15 +1,18 @@
-from __future__ import unicode_literals, print_function
+from __future__ import annotations
+
 import csv
 import logging
 import os
 import sys
 import platform
 import shlex
+from typing import Any, cast
 
 
 from litecli import __version__
 from litecli.packages.special import iocommands
 from .main import special_command, RAW_QUERY, PARSED_QUERY
+from .types import DBCursor
 
 log = logging.getLogger(__name__)
 
@@ -22,9 +25,14 @@ log = logging.getLogger(__name__)
     case_sensitive=True,
     aliases=("\\dt",),
 )
-def list_tables(cur, arg=None, arg_type=PARSED_QUERY, verbose=False):
+def list_tables(
+    cur: DBCursor,
+    arg: str | None = None,
+    arg_type: int = PARSED_QUERY,
+    verbose: bool = False,
+) -> list[tuple]:
     if arg:
-        args = ("{0}%".format(arg),)
+        args: tuple[str, ...] = ("{0}%".format(arg),)
         query = """
             SELECT name FROM sqlite_master
             WHERE type IN ('table','view') AND name LIKE ? AND name NOT LIKE 'sqlite_%' AND name NOT LIKE 'sqlean_%'
@@ -64,9 +72,14 @@ def list_tables(cur, arg=None, arg_type=PARSED_QUERY, verbose=False):
     case_sensitive=True,
     aliases=("\\dv",),
 )
-def list_views(cur, arg=None, arg_type=PARSED_QUERY, verbose=False):
+def list_views(
+    cur: DBCursor,
+    arg: str | None = None,
+    arg_type: int = PARSED_QUERY,
+    verbose: bool = False,
+) -> list[tuple]:
     if arg:
-        args = ("{0}%".format(arg),)
+        args: tuple[str, ...] = ("{0}%".format(arg),)
         query = """
             SELECT name FROM sqlite_master
             WHERE type = 'view' AND name LIKE ? AND name NOT LIKE 'sqlite_%' AND name NOT LIKE 'sqlean_%'
@@ -97,9 +110,9 @@ def list_views(cur, arg=None, arg_type=PARSED_QUERY, verbose=False):
     arg_type=PARSED_QUERY,
     case_sensitive=True,
 )
-def show_schema(cur, arg=None, **_):
+def show_schema(cur: DBCursor, arg: str | None = None, **_: Any) -> list[tuple]:
     if arg:
-        args = (arg,)
+        args: tuple[str, ...] = (arg,)
         query = """
             SELECT sql FROM sqlite_master
             WHERE tbl_name==? AND sql IS NOT NULL
@@ -133,7 +146,7 @@ def show_schema(cur, arg=None, **_):
     case_sensitive=True,
     aliases=("\\l",),
 )
-def list_databases(cur, **_):
+def list_databases(cur: DBCursor, **_: Any) -> list[tuple]:
     query = "PRAGMA database_list"
     log.debug(query)
     cur.execute(query)
@@ -152,9 +165,14 @@ def list_databases(cur, **_):
     case_sensitive=True,
     aliases=("\\di",),
 )
-def list_indexes(cur, arg=None, arg_type=PARSED_QUERY, verbose=False):
+def list_indexes(
+    cur: DBCursor,
+    arg: str | None = None,
+    arg_type: int = PARSED_QUERY,
+    verbose: bool = False,
+) -> list[tuple]:
     if arg:
-        args = ("{0}%".format(arg),)
+        args: tuple[str, ...] = ("{0}%".format(arg),)
         query = """
             SELECT name, sql FROM sqlite_master
             WHERE type = 'index' AND tbl_name LIKE ? AND name NOT LIKE 'sqlite_%'
@@ -187,7 +205,7 @@ def list_indexes(cur, arg=None, arg_type=PARSED_QUERY, verbose=False):
     aliases=("\\s",),
     case_sensitive=True,
 )
-def status(cur, **_):
+def status(cur: DBCursor, **_: Any) -> list[tuple]:
     # Create output buffers.
     footer = []
     footer.append("--------------")
@@ -204,9 +222,8 @@ def status(cur, **_):
     query = "SELECT file from pragma_database_list() where name = 'main';"
     log.debug(query)
     cur.execute(query)
-    db = cur.fetchone()[0]
-    if db is None:
-        db = ""
+    row = cur.fetchone()
+    db = row[0] if row else ""
 
     footer.append("Current database: " + db)
     if iocommands.is_pager_enabled():
@@ -229,7 +246,7 @@ def status(cur, **_):
     arg_type=PARSED_QUERY,
     case_sensitive=True,
 )
-def load_extension(cur, arg, **_):
+def load_extension(cur: DBCursor, arg: str, **_: Any) -> None:
     args = shlex.split(arg)
     if len(args) != 1:
         raise TypeError(".load accepts exactly one path")
@@ -237,7 +254,6 @@ def load_extension(cur, arg, **_):
     conn = cur.connection
     conn.enable_load_extension(True)
     conn.load_extension(path)
-    return [(None, None, None, "")]
 
 
 @special_command(
@@ -248,13 +264,13 @@ def load_extension(cur, arg, **_):
     case_sensitive=True,
     aliases=("\\d", "desc"),
 )
-def describe(cur, arg, **_):
+def describe(cur: DBCursor, arg: str | None, **_: Any) -> list[tuple]:
     if arg:
         query = """
             PRAGMA table_info({})
         """.format(arg)
     else:
-        return list_tables(cur)
+        return cast(list[tuple[Any, ...]], list_tables(cur))
 
     log.debug(query)
     cur.execute(query)
@@ -275,8 +291,8 @@ def describe(cur, arg, **_):
     arg_type=PARSED_QUERY,
     case_sensitive=True,
 )
-def import_file(cur, arg=None, **_):
-    def split(s):
+def import_file(cur: DBCursor, arg: str | None = None, **_: Any) -> list[tuple]:
+    def split(s: str) -> list[str]:
         # this is a modification of shlex.split function, just to make it support '`',
         # because table name might contain '`' character.
         lex = shlex.shlex(s, posix=True)
@@ -285,6 +301,8 @@ def import_file(cur, arg=None, **_):
         lex.quotes += "`"
         return list(lex)
 
+    if arg is None:
+        raise TypeError("Usage: .import filename table")
     args = split(arg)
     log.debug("[arg = %r], [args = %r]", arg, args)
     if len(args) != 2:
