@@ -1,9 +1,10 @@
-from __future__ import unicode_literals
+from __future__ import annotations
 import logging
 from collections import namedtuple
+from enum import Enum
+from typing import Any, Callable, cast
 
 from . import export
-from enum import Enum
 
 log = logging.getLogger(__name__)
 
@@ -46,11 +47,11 @@ class Verbosity(Enum):
 
 
 @export
-def parse_special_command(sql):
+def parse_special_command(sql: str) -> tuple[str, "Verbosity", str]:
     """
     Parse a special command, extracting the base command name, verbosity
     (normal, verbose (+), or succinct (-)), and the remaining argument.
-    Mirrors mycli's behavior.
+    Mirrors the behavior used in similar CLI tools.
     """
     command, _, arg = sql.partition(" ")
     verbosity = Verbosity.NORMAL
@@ -64,15 +65,15 @@ def parse_special_command(sql):
 
 @export
 def special_command(
-    command,
-    shortcut,
-    description,
-    arg_type=PARSED_QUERY,
-    hidden=False,
-    case_sensitive=False,
-    aliases=(),
-):
-    def wrapper(wrapped):
+    command: str,
+    shortcut: str,
+    description: str,
+    arg_type: int = PARSED_QUERY,
+    hidden: bool = False,
+    case_sensitive: bool = False,
+    aliases: tuple[str, ...] = (),
+) -> Callable:
+    def wrapper(wrapped: Callable) -> Callable:
         register_special_command(
             wrapped,
             command,
@@ -90,15 +91,15 @@ def special_command(
 
 @export
 def register_special_command(
-    handler,
-    command,
-    shortcut,
-    description,
-    arg_type=PARSED_QUERY,
-    hidden=False,
-    case_sensitive=False,
-    aliases=(),
-):
+    handler: Callable,
+    command: str,
+    shortcut: str,
+    description: str,
+    arg_type: int = PARSED_QUERY,
+    hidden: bool = False,
+    case_sensitive: bool = False,
+    aliases: tuple[str, ...] = (),
+) -> None:
     cmd = command.lower() if not case_sensitive else command
     COMMANDS[cmd] = SpecialCommand(handler, command, shortcut, description, arg_type, hidden, case_sensitive)
     for alias in aliases:
@@ -115,7 +116,7 @@ def register_special_command(
 
 
 @export
-def execute(cur, sql):
+def execute(cur: Any, sql: str) -> list[tuple[Any, ...]]:
     """Execute a special command and return the results. If the special command
     is not supported a KeyError will be raised.
     """
@@ -132,15 +133,20 @@ def execute(cur, sql):
             raise CommandNotFound("Command not found: %s" % command)
 
     if special_cmd.arg_type == NO_QUERY:
-        return special_cmd.handler()
+        return cast(list[tuple[Any, ...]], special_cmd.handler())
     elif special_cmd.arg_type == PARSED_QUERY:
-        return special_cmd.handler(cur=cur, arg=arg, verbose=(verbosity == Verbosity.VERBOSE))
+        return cast(
+            list[tuple[Any, ...]],
+            special_cmd.handler(cur=cur, arg=arg, verbose=(verbosity == Verbosity.VERBOSE)),
+        )
     elif special_cmd.arg_type == RAW_QUERY:
-        return special_cmd.handler(cur=cur, query=sql)
+        return cast(list[tuple[Any, ...]], special_cmd.handler(cur=cur, query=sql))
+
+    raise CommandNotFound(f"Command type not found: {command}")
 
 
 @special_command("help", "\\?", "Show this help.", arg_type=NO_QUERY, aliases=("\\?", "?"))
-def show_help():  # All the parameters are ignored.
+def show_help() -> list[tuple]:  # All the parameters are ignored.
     headers = ["Command", "Shortcut", "Description"]
     result = []
 
@@ -152,7 +158,7 @@ def show_help():  # All the parameters are ignored.
 
 @special_command(".exit", "\\q", "Exit.", arg_type=NO_QUERY, aliases=("\\q", "exit"))
 @special_command("quit", "\\q", "Quit.", arg_type=NO_QUERY)
-def quit(*_args):
+def quit(*_args: Any) -> None:
     raise EOFError
 
 
@@ -178,5 +184,5 @@ def quit(*_args):
     case_sensitive=False,
     aliases=(".ai", ".llm"),
 )
-def stub():
+def stub() -> None:
     raise NotImplementedError
