@@ -1,4 +1,7 @@
+from __future__ import annotations
+
 import logging
+from typing import Any, Generator, Iterable
 
 from contextlib import closing
 
@@ -14,6 +17,7 @@ from litecli.packages.special.utils import check_if_sqlitedotcommand
 
 import sqlparse
 import os.path
+from urllib.parse import urlparse
 
 from .packages import special
 
@@ -55,25 +59,36 @@ class SQLExecute(object):
     functions_query = '''SELECT ROUTINE_NAME FROM INFORMATION_SCHEMA.ROUTINES
     WHERE ROUTINE_TYPE="FUNCTION" AND ROUTINE_SCHEMA = "%s"'''
 
-    def __init__(self, database):
-        self.dbname = database
-        self._server_type = None
-        self.conn = None
+    def __init__(self, database: str | None):
+        self.dbname: str | None = database
+        self._server_type: tuple[str, str] | None = None
+        # Connection can be sqlite3.Connection or sqlean.sqlite3 connection.
+        self.conn: Any | None = None
         if not database:
             _logger.debug("Database is not specified. Skip connection.")
             return
         self.connect()
 
-    def connect(self, database=None):
+    def connect(self, database: str | None = None) -> None:
         db = database or self.dbname
         _logger.debug("Connection DB Params: \n\tdatabase: %r", db)
+        if db is None:
+            # Nothing to connect to.
+            return
 
-        db_name = os.path.expanduser(db)
-        db_dir_name = os.path.dirname(os.path.abspath(db_name))
-        if not os.path.exists(db_dir_name):
-            raise Exception("Path does not exist: {}".format(db_dir_name))
+        location = urlparse(db)
+        if location.scheme and location.scheme == "file":
+            uri = True
+            db_name = db
+            db_filename = location.path
+        else:
+            uri = False
+            db_filename = db_name = os.path.expanduser(db)
+            db_dir_name = os.path.dirname(os.path.abspath(db_filename))
+            if not os.path.exists(db_dir_name):
+                raise Exception("Path does not exist: {}".format(db_dir_name))
 
-        conn = sqlite3.connect(database=db_name, isolation_level=None)
+        conn = sqlite3.connect(database=db_name, isolation_level=None, uri=uri)
         conn.text_factory = lambda x: x.decode("utf-8", "backslashreplace")
         if self.conn:
             self.conn.close()
@@ -81,9 +96,9 @@ class SQLExecute(object):
         self.conn = conn
         # Update them after the connection is made to ensure that it was a
         # successful connection.
-        self.dbname = db
+        self.dbname = db_filename
 
-    def run(self, statement):
+    def run(self, statement: str) -> Iterable[tuple]:
         """Execute the sql in the database and return the results. The results
         are a list of tuples. Each tuple has 4 values
         (title, rows, headers, status).
@@ -135,10 +150,11 @@ class SQLExecute(object):
                     yield ("dot command not implemented", None, None, None)
                 else:
                     _logger.debug("Regular sql statement. sql: %r", sql)
+                    assert cur is not None
                     cur.execute(sql)
                     yield self.get_result(cur)
 
-    def get_result(self, cursor):
+    def get_result(self, cursor: Any) -> tuple[str | None, list | None, list | None, str]:
         """Get the current result's data from the cursor."""
         title = headers = None
 
@@ -162,24 +178,25 @@ class SQLExecute(object):
 
         return (title, cursor, headers, status)
 
-    def tables(self):
+    def tables(self) -> Generator[tuple[str], None, None]:
         """Yields table names"""
-
+        assert self.conn is not None
         with closing(self.conn.cursor()) as cur:
             _logger.debug("Tables Query. sql: %r", self.tables_query)
             cur.execute(self.tables_query)
             for row in cur:
                 yield row
 
-    def table_columns(self):
+    def table_columns(self) -> Generator[tuple[str, str], None, None]:
         """Yields column names"""
+        assert self.conn is not None
         with closing(self.conn.cursor()) as cur:
             _logger.debug("Columns Query. sql: %r", self.table_columns_query)
             cur.execute(self.table_columns_query)
             for row in cur:
                 yield row
 
-    def databases(self):
+    def databases(self) -> Generator[str, None, None]:
         if not self.conn:
             return
 
@@ -188,27 +205,15 @@ class SQLExecute(object):
             for row in cur.execute(self.databases_query):
                 yield row[1]
 
-    def functions(self):
+    def functions(self) -> Iterable[tuple]:
         """Yields tuples of (schema_name, function_name)"""
-
+        assert self.conn is not None
         with closing(self.conn.cursor()) as cur:
             _logger.debug("Functions Query. sql: %r", self.functions_query)
             cur.execute(self.functions_query % self.dbname)
             for row in cur:
                 yield row
 
-    def show_candidates(self):
-        with closing(self.conn.cursor()) as cur:
-            _logger.debug("Show Query. sql: %r", self.show_candidates_query)
-            try:
-                cur.execute(self.show_candidates_query)
-            except sqlite3.DatabaseError as e:
-                _logger.error("No show completions due to %r", e)
-                yield ""
-            else:
-                for row in cur:
-                    yield (row[0].split(None, 1)[-1],)
-
-    def server_type(self):
+    def server_type(self) -> tuple[str, str]:
         self._server_type = ("sqlite3", "3")
         return self._server_type
